#!/usr/bin/env nextflow

nextflow.enable.dsl=2

/* ChIPseq pipeline :
 * Pipeline that analyses ChIPseq data 
 * Preprocessing, filtration, alignment, peak calling...
*/

/*
 ****************************************************************
                       parameters
 ****************************************************************
*/

params.paired_end = false
/* false for single end data, true for paired-end data

@type: Boolean
*/

params.fastq = "./data/tiny-delta-te-dataset/fastq_rnaseq/*.gz"
/* Fastq files

@type: Files
*/

params.genome = "./data/tiny-delta-te-dataset/synth.fasta"
/* A genome file

@type: File
*/

params.chrom_sizes = "./data/tiny-delta-te-dataset/chrom.sizes"
/* samtools generated genome.sizes file: samtools faidx synth.fasta and cut -f 1,2 synth.fasta.fai > chrom.sizes

@type: File
*/

// params.idx = ""
/* already indexed reference genome ? enter path...

@Type: String
*/

/* Parametres ctrl names & IP names
utiliser l'oppérateur .join ou .filter

/* Params project Name */
params.project = ""

/* Params out */
params.fastp_out = "$params.project/fastp/"
params.index_fasta_out = "$params.project/Indexed_genome/"
params.sort_bam_out = "$params.project/Bam_filtered_sorted/"
params.index_bam_out = "$params.project/Bam_filt_sort_indexed/"
params.bam_to_bigwig_out = "$params.project/BigWig/" 
params.peak_calling_bg_out = "$params.project/Peak_calling/"
params.bam_to_bed_out = "$params.project/Bed/"
params.bed_slop_out = "$params.project/Bed_sloped/"
params.bedGraph_out = "$params.project/BedGraph/"
params.chipseq_bam2BG_out = "$params.project/chipseq_BigGig"

/*
 ****************************************************************
                              Logs
 ****************************************************************
*/

log.info "fastq files : ${params.fastq}"
log.info "genome file : ${params.genome}"
log.info "genome sizes : ${params.chrom_sizes}"
/* log.info "indexed genome file : ${params.idxgenome}" */

/*
log.info "paired-end data: ${params.paired_end}"
log.info "output folder results/${params.folder}"
*/

/*
 ****************************************************************
                  Channel definitions
 ****************************************************************
*/

/* Raw paired-end reads fastq */
if (params.paired_end) {
  Channel
    .fromFilePairs( params.fastq, size: 2 ) //def une error
    .set { fastq_files }
} else {
  Channel
    .fromPath( params.fastq )
    .ifEmpty { error "Cannot find any files matching: ${params.fastq}" }
    .map{it -> [(it.baseName =~ /([^\.]*)/)[0][1], [it] ]}
    .set{ fastq_files }
}

/*
if (params.idx != "") {
  Channel
    .fromPath( params.idx )
    .ifEmpty { error "Cannot find idexed genome reference files matching: ${params.idx}" }
    .map { it -> [(it.baseName =~ /([^\.]*)/)[0][1], [it] ]}
    .set { index_fasta.out }
}
*/

Channel
  .fromPath( params.genome )
  .ifEmpty { error "Cannot find any files matching: ${params.genome}" }
  .map{it -> [(it.baseName =~ /([^\.]*)/)[0][1], it]}
  .set { genome_file }

/* 
Channel // IP & CTRL names
  .from( params.ctrl )
  
Channel 
  .from( params.ip )
*/

/*
if...
Channel
  .fromPath( params.idxgenome )
  .set { genome_idx }
*/

Channel
  .fromPath( params.chrom_sizes )
  .ifEmpty { error "Cannot find any files matching: ${params.chrom_sizes}" }
  .map{it -> [(it.baseName =~ /([^\.]*)/)[0][1], it]}
  .set{ genome_sizes }

/*
 ****************************************************************
                          Imports
 ****************************************************************
*/

fastqc_mod = "./nf_modules/fastqc/main.nf"
include { fastp_default } from "./nf_modules/fastp/main.nf"
include { fastqc_fastq as fastqc_raw } from fastqc_mod addParams(fastqc_fastq_out: "$params.project/01_fastqc_raw/")
include { fastqc_fastq as fastqc_preprocessed } from fastqc_mod addParams(fastqc_fastq_out: "$params.project/02_fastqc_preprocessed/")
include { multiqc } from './nf_modules/multiqc/main.nf' addParams(multiqc_out: "$params.project/QC/")
include { index_fasta } from "./nf_modules/bowtie2/main.nf"
include { mapping_fastq } from "./nf_modules/bowtie2/main.nf"
include { filter_bam_quality } from "./nf_modules/samtools/main.nf"
include { sort_bam } from "./nf_modules/samtools/main.nf"
include { index_bam } from "./nf_modules/samtools/main.nf"
include { bam_to_bigwig } from "./nf_modules/deeptools/main.nf"
include { chipseq_bam2BG } from "./nf_modules/deeptools/main.nf"
include { peak_calling_bg } from "./nf_modules/macs3/main.nf"
include { bam_to_bed } from "./nf_modules/bedtools/main.nf"
include { bed_slop } from "./nf_modules/bedtools/main.nf"
include { bed_to_bedGraph } from "./nf_modules/bedtools/main.nf"

/*
 ****************************************************************
                          Workflow
 ****************************************************************
*/

workflow {

  // fastp
  fastp_default(fastq_files)
  
  // fastqc_rawdata
  fastqc_raw(fastq_files)
  // fastqc_processed
  fastqc_preprocessed(fastp_default.out.fastq)
  // multiqc
  multiqc(
    fastqc_raw.out.report
    .mix(
      fastqc_preprocessed.out.report
      ).collect()
  )

  // index reference genome
  index_fasta(genome_file)
  // mapping preprocessed reads
  mapping_fastq(index_fasta.out.index.collect(), fastp_default.out.fastq)
 
  /*if (!params.idxgenome) {
    index_fasta(genome_file)
    mapping_fastq(index_fasta.out.index.collect(), fastp_default.out.fastq)
  } else {
    mapping_fastq(genome_idx.collect(), fastp_default.out.fastq)
  }
  */

  // filter bam - remove reads with quality <30
  filter_bam_quality(mapping_fastq.out.bam)

  // samtools_sort
  sort_bam(filter_bam_quality.out.bam)

  // samtools_index
  index_bam(sort_bam.out.bam)

  // Create a bigwig file
  // bam_to_bigwig(index_bam.out.bam_idx)

  // Chipseq Bam 2 bigwig file with reads extends
  chipseq_bam2BG(index_bam.out.bam_idx)

  // From Bam to Bed
  // bam_to_bed(sort_bam.out.bam)

  // Extension of reads with bedtools slop
  // bed_slop(bam_to_bed.out.bed, genome_sizes.collect())

  // From bed to bedgraph
  // bed_to_bedGraph(bed_slop.out, genome_sizes.collect())

  // peak calling using MACS3 Prend des bed ou des bam en entrée...
  // peak_calling_bg()
}